var vows   = require('vows')
  , assert = require('assert')
  , express = require('express')
  , request = require('request');
//  , provision = require('../');

var app = express.createServer();

var provision = require('../');

//Configuration
app.configure(function(){
  app.use(express.bodyParser());
  app.use(express.cookieParser());
  app.use(express.session({ secret: "simyungk" }));
  app.use(express.methodOverride());

  // use provision middleware
  app.use(provision.middleware());
});


var _req; // temp

//Routes
app.get('/', function(req, res){
  _req = req; // to test...
  var prov = req.session.provision;
  res.send(JSON.stringify(prov), 200);
});

app.listen(3000);
console.log("Express server listening on port %d in %s mode", app.address().port, app.settings.env);

function testRequest(callback) {
  console.log('----------------------------------');
  console.log('try request...');
  request({
    uri: 'http://localhost:3000',
    method: 'GET',
    headers: {
      'accept-language': 'ko-KR, en-US', 
      'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2'
    }
  }, callback);
}

vows.describe('provision test').addBatch({
  "enable ": {
    topic: function () {
      provision.options = {
        monomi: false,
        useragent: false,
        geoip: false,
        language: false
      };
      testRequest(this.callback);
    },
    "modules not loaded": function (err, res, body) {
      assert.equal(typeof provision.mod.monomi, 'undefined');
      assert.equal(typeof provision.mod.useragent, 'undefined');
      assert.equal(typeof provision.mod.geoip, 'undefined');
    },
    "provision not created": function (err, res, body) {
      assert.equal(typeof _req.session.provision, 'undefined');
    },
    "session clear": function (err, res, body) {
      delete _req.session.provision;
      provision.mod = {};
    },
    "language": {
      topic: function () {
        provision.options.language = true;
        testRequest(this.callback);
      },
      "modules not loaded": function (err, res, body) {
        assert.equal(typeof provision.mod.monomi, 'undefined');
        assert.equal(typeof provision.mod.useragent, 'undefined');
        assert.equal(typeof provision.mod.geoip, 'undefined');
      },
      "provision created": function (err, res, body) {
        assert.equal(typeof _req.session.provision != 'undefined', true);
        assert.equal(_req.session.provision.locale, 'ko');
        assert.equal(_req.session.provision.country, 'kr');
        assert.equal(_req.session.provision.browserType, undefined);
        assert.equal(_req.session.provision.family, undefined);
        assert.equal(_req.session.provision.location, undefined);
      },
      "clear": function (err, res, body) {
        delete _req.session.provision;
        provision.mod = {};
      },
      "monomi": {
        topic: function () {
          provision.options.monomi = true;
          testRequest(this.callback);
        },
        "monomi module loaded only": function (err, res, body) {
          assert.equal(typeof provision.mod.monomi != 'undefined', true);
          assert.equal(typeof provision.mod.useragent, 'undefined');
          assert.equal(typeof provision.mod.geoip, 'undefined');
        },
        "provision created": function (err, res, body) {
          assert.equal(typeof _req.session.provision != 'undefined', true);
          assert.equal(_req.session.provision.browserType != 'undefined', true);
        },
        "clear": function (err, res, body) {
          delete _req.session.provision;
          provision.mod = {};
        },
        "useragent": {
          topic: function () {
            provision.options.useragent = true;
            testRequest(this.callback);
          },
          "monomi, useragent modules loaded": function (err, res, body) {
            assert.equal(typeof provision.mod.monomi != 'undefined', true);
            assert.equal(typeof provision.mod.useragent != 'undefined', true);
            assert.equal(typeof provision.mod.geoip, 'undefined');
          },
          "provision created": function (err, res, body) {
            assert.equal(typeof _req.session.provision != 'undefined', true);
            assert.equal(_req.session.provision.family != 'undefined', true);
            assert.equal(_req.session.provision.os != 'undefined', true);
          },
          "clear": function (err, res, body) {
            delete _req.session.provision;
            provision.mod = {};
          },
          "geoip": {
            topic: function () {
              provision.options.geoip = true;
              testRequest(this.callback);
            },
            "monomi, useragent, geoip modules loaded": function (err, res, body) {
              if (typeof provision.mod.geoip == 'undefined') {
                console.log('skip geoip test');
              }
              assert.equal(typeof provision.mod.useragent != 'undefined', true);
            },
            "provision created": function (err, res, body) {
              assert.equal(typeof _req.session.provision != 'undefined', true);
              if (provision.mod.geoip) {
                assert.equal(_req.session.provision.location != 'undefined', true);
              }
            },
            "canProvisioning returns false": function (err, res, body) {
              assert.equal(provision._steps.canProvisioning(_req, provision.options), false);
            },
            "clear": function (err, res, body) {
              delete _req.session.provision;
              provision.mod = {};
            }
          }
        }
      }
    }
  }
}).export(module);

//vows.describe('provision test').addBatch({
//  "When using your request": {
//    "try to provision information": {
//      "case: invalid ip address": {
//        topic: function () {
//          request({
//            uri: 'http://localhost:3000',
//            method: 'GET',
//            headers: {
//              'accept-language': 'ko-KR, en-US', 
//              'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2'
//            }
//          }, this.callback);
//        },
//        "localization: locale, region": function (err, res, body) {
//          var prov = JSON.parse(body);
//          assert.equal(prov.locale, 'ko');
//          assert.equal(prov.region, 'kr');
//        },
//        "useragent: family, os, type": function (err, res, body) {
//          var prov = JSON.parse(body);
//          assert.equal(prov.locale, 'ko');
//          assert.equal(prov.region, 'kr');
//        },
//        "Print Locale, Region": function (err, res, body) {
////          console.log(JSON.stringify(_req.session.provision));
//        }
//      }
//    }
//  }
//}).export(module);

